% Yanyu Liang
%
% Copyright (C) 2013-2016 Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

classdef TestSlml2Img < matlab.unittest.TestCase
    properties
    end
    
    methods (Test)
        function test_empty_model_list( testCase )
            disp('test001: must return false when list of models is empty')
            list_of_models = {};
            param = [];
            answer = slml2img( list_of_models, param );
            testCase.verifyFalse( answer );
            temp = [ pwd filesep 'demo' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            temp = [ pwd filesep 'log' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            
            temp = [ pwd filesep 'demo1.tif' ];
            if exist( temp )
                delete( temp );
            end
            
            temp = [ pwd filesep 'temp' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
        end
        
        function test_non_cell_array_model_list( testCase )
            disp('test002: must return false when list of models is not a cell array')
            list_of_models = 1;
            param = [];
            answer = slml2img( list_of_models, param );
            testCase.verifyFalse( answer );
            
            list_of_models = rand(3);
            param = [];
            answer = slml2img( list_of_models, param );
            testCase.verifyFalse( answer );
            temp = [ pwd filesep 'demo' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            temp = [ pwd filesep 'log' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            
            temp = [ pwd filesep 'demo1.tif' ];
            if exist( temp )
                delete( temp );
            end
            
            temp = [ pwd filesep 'temp' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
        end
        
        function test_non_existing_file_in_model_list( testCase )
            disp('Must return false if the list of files contains a non existing file')
            list_of_models = {'non-existing-file.mat'};
            param = [];
            answer = slml2img( list_of_models, param );
            testCase.verifyFalse( answer );
            temp = [ pwd filesep 'demo' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            temp = [ pwd filesep 'log' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            
            temp = [ pwd filesep 'demo1.tif' ];
            if exist( temp )
                delete( temp );
            end
            
            temp = [ pwd filesep 'temp' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
        end
        
        function test_default_values_from_param( testCase )
            disp('test004: check default values from parameter structure')
            list_of_models = {'/models/2D/endosome.mat'};
            param.debug = true;
            answer = slml2img( list_of_models, param );
            testCase.verifyTrue( answer );
            
            load( [ pwd filesep 'temp' filesep 'param.mat' ] );
            testCase.verifyTrue( param.debug );
            testCase.verifyFalse( param.display );
            
            clear param
            temp = [ pwd filesep 'demo' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            temp = [ pwd filesep 'log' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            
            temp = [ pwd filesep 'demo1.tif' ];
            if exist( temp )
                delete( temp );
            end
            
            temp = [ pwd filesep 'temp' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
        end
        
        function test_checking_models_have_same_dimensionality( testCase )
            disp('test005: check that models have the same dimensionality')
            list_of_models = {'/models/2D/endosome.mat', ...
                '/models/3D/lamp2.mat'};
            param.debug = true;
            answer = slml2img( list_of_models, param );
            testCase.verifyFalse( answer );
            
            tic
            list_of_models = {'/models/2D/endosome.mat', ...
                '/models/2D/lysosome.mat'};
            param.debug = true;
            answer = slml2img( list_of_models, param );
            toc
            testCase.verifyTrue( answer );
            temp = [ pwd filesep 'demo' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            temp = [ pwd filesep 'log' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            
            temp = [ pwd filesep 'demo1.tif' ];
            if exist( temp )
                delete( temp );
            end
            
            temp = [ pwd filesep 'temp' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
        end
        
        function test_checking_the_existence_of_output_2D( testCase )
            disp('test006: check the existence of output files for 2D model')
            list_of_models = {'/models/2D/nucleolus.mat', ...
                '/models/2D/endosome.mat', ...
                '/models/2D/mitochondrion.mat', ...
                '/models/2D/lysosome.mat'};
            param.debug = true;
            param.output.tifimages=true;
            param.output.indexedimage=true;
            param.output.blenderfile=true;
            answer = slml2img( list_of_models, param );
            
            
            testCase.verifyTrue( check({'demo1.tif'}) );
            
            temp = [ pwd filesep 'demo' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            temp = [ pwd filesep 'log' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            
            temp = [ pwd filesep 'demo1.tif' ];
            if exist( temp )
                delete( temp );
            end
            
            temp = [ pwd filesep 'temp' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
        end
        
        function test_checking_the_existence_of_output_3D( testCase )
            disp('test007: check the existence of output files for 3D model')
            list_of_models ={'/models/3D/lamp2.mat'};
            param.debug = true;
            param.output.tifimages=true;
            param.output.indexedimage=true;
            param.output.blenderfile=true;
            param.output.blender.downsample=1;
            answer = slml2img( list_of_models, param );
            
            outdir=['demo' filesep 'cell1' filesep];
            indexedtif=[outdir 'indexed.tif'];
            tif=[outdir 'nucleus.tif'];
            model=load(list_of_models{1});
            objfile=[outdir model.model.proteinModel.class '.obj'];
            testCase.verifyTrue( check({indexedtif,tif,objfile}) );
            temp = [ pwd filesep 'demo' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            temp = [ pwd filesep 'log' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            
            temp = [ pwd filesep 'demo1.tif' ];
            if exist( temp )
                delete( temp );
            end
            
            temp = [ pwd filesep 'temp' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
        end
        
        function teardownOnce( testCase )
            temp = [ pwd filesep 'demo' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            temp = [ pwd filesep 'log' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
            
            
            temp = [ pwd filesep 'demo1.tif' ];
            if exist( temp )
                delete( temp );
            end
            
            temp = [ pwd filesep 'temp' ];
            if exist( temp )
                rmdir( temp, 's' );
            end
        end
    end
end