function [ model ] = param2model_2D( cell_params, options )
%Initial draft for param2model_2D.
%
%Eventually we would like an aribitrary model specified in options, and
%this function navigates the model structure and constructs it in the
%correct order.

if (isfield( options, 'nucleus') && isfield( options.nucleus, 'type') && strcmpi(options.nucleus.type, 'diffeomorphic')) ...
   || (isfield( options, 'cell')   && isfield( options.cell, 'type')    && strcmpi(options.cell.type, 'diffeomorphic'))
    isdiffeomorphic = true; 
else
    isdiffeomorphic = false;
end

%if set to nuc, build only nuc model

%loop through parameterizations until we find one that exists
c = 1;
breakflag = false;
while ~breakflag
    if iscell(cell_params(c)) && ischar(cell_params{c}) && exist(cell_params{c}, 'file')
        cell_param = load(cell_params{c}, 'options');
        breakflag = true;
    elseif iscell(cell_params(c)) && isstruct(cell_params{c})
        cell_param = cell_params{c};
        breakflag = true;
    elseif isstruct(cell_params(c))
        cell_param = cell_params(c);
        breakflag = true;
    end
    c = c+1;
end


%%Cell and Nuclear Shape Model
if isdiffeomorphic
    components = {};
    
    if (isfield( options, 'cell')   && isfield( options.cell, 'type')    && strcmpi(options.cell.type, 'diffeomorphic'))
        % xruan 01/05/2016 change components{end} = 'cell'; to components{end + 1} = 'cell';
        components{end + 1} = 'cell';
    end
    
    if (isfield( options, 'nucleus') && isfield( options.nucleus, 'type') && strcmpi(options.nucleus.type, 'diffeomorphic'))
        % xruan 01/05/2016 change components{end} = 'nucleus'; to components{end + 1} = 'nucleus';
        components{end + 1} = 'nuc';
    end
    
    % xruan 01/05/2016 change component to components to unify the name 
    % make options_img as a single structure 
    options_img = struct('components', {components}, 'format', 'indexed');
    options.imfunc = @(x) param2img(cell_params{x}, options_img);
    % options.imfunc(1);
    
    model.cellShapeModel = train_diffeomorphic_model(options);
    model.nuclearShapeModel = model.cellShapeModel;
else
    if isfield(options, 'nucleus')
        nuc_model_save = [options.paramdir filesep 'nuc.mat'];
        if ~exist(nuc_model_save, 'file')
            switch options.nucleus.type
                case 'medial axis'
                    options.model = ml_initparam(options.model, struct('nucleus', []));
                    options.model.nucleus = ml_initparam(options.model.nucleus, ...
                            struct('modelname','mxp','constknots',{{0.5,0.5}}));
                        
                    param_tmp = loadFiles(cell_params, 'nuc');
                    param_tmp = param_tmp(~cellfun(@isempty, param_tmp));
                    
                    component_param = cellfun(@(x) x.nuc, param_tmp, 'UniformOutput', false);

                    nuclear_model = ml_trainshapemodel2D( component_param, options.model.nucleus );
                    
%                     nuclear_model = param2cylsurf(cell_params, options);
                otherwise
                    warning(['Unsupported cell model type ' options.nucleus.type '. Returning empty model.'])
                    nuclear_model = [];
            end
            save(nuc_model_save, 'nuclear_model')
        else
            load(nuc_model_save)
        end
        
        model.nuclearShapeModel = nuclear_model;
        model.nuclearShapeModel.resolution = cell_param.options.model.resolution;
    end
            
    if isfield(options, 'cell')
        cell_model_save = [options.paramdir filesep 'cell.mat'];
        if ~exist(cell_model_save, 'file')
            switch options.cell.type
                case 'ratio'
                    param_tmp = loadFiles(cell_params, 'cell');
                    param_tmp = param_tmp(~cellfun(@isempty, param_tmp));
                    
                    component_param = cellfun(@(x) x.cell, param_tmp, 'UniformOutput', false);
                    
                    cell_model = param2cell_ratio_model_2D(component_param, options);
                otherwise 
                    warning(['Unsupported cell model type ' options.cell.type '. Returning empty model.'])
                    cell_model = [];
            end
            
            save(cell_model_save, 'cell_model')
        else
            load(cell_model_save)
        end
        model.cellShapeModel = cell_model;
        model.cellShapeModel.resolution = cell_param.options.model.resolution;
    end
end

%%Protein model
if isfield(options, 'protein') && isfield(options.protein, 'type') 
    protein_model_save = [options.paramdir filesep options.protein.type '.mat'];
    if ~exist(protein_model_save, 'file')
        
        switch options.protein.type
            case 'vesicle'
                param_tmp = loadFiles(cell_params, {'prot', 'cell', 'imsize'});
                param_tmp = param_tmp(~cellfun(@isempty, param_tmp));
                
                component_param_prot = cellfun(@(x) x.prot, param_tmp, 'UniformOutput', false);
                component_param_cell = cellfun(@(x) x.cell, param_tmp, 'UniformOutput', false);
                imsizes = cellfun(@(x) x.imsize, param_tmp, 'UniformOutput', false);
                imsizes = vertcat(imsizes{:});
                
                protein_model = param2protein_gaussian_model_2D(component_param_prot, component_param_cell, imsizes, options);
            case 'network'
                warning(['Unsupported protein model type "network" is currently not supported. Returning empty model.'])
%                 protein_model = param2microtubule_model_3D(cell_params, options);
            otherwise
                 warning(['Unsupported protein model type ' options.protein.type '. Returning empty model.'])
                 protein_model = [];
        end
        save(protein_model_save, 'protein_model')
    else
        load(protein_model_save)
    end

    model.proteinModel = protein_model;
    model.proteinModel.dimensionality = '2D';
    model.proteinModel.resolution = cell_param.options.model.resolution;
end
    


end

