function param = img2param_3D( imdna_path,imcell_path,...
    improt_path,immask_path, savedir, options )
%Parameterizes an image of a cell for use in the CellOrganizer suite.
%
%Inputs
%imdna_path - cell array of image files for dna parameter extraction
%imcell_path - cell array of image files for cell parameter extraction
%                   (optional if options.train.flag = 'nuclear')  
%improt_path - cell array of image files for protein parameter extraction
%                   (optional if options.train.flag = 'nuclear' or 'framework')
%immask_path - cell array of image files for masking a single cell
%                   from a given image (these are always optional and can
%                   be passed in as an array of [])
%options             a structure holding possible parameter options. This
%                  struct should also contain all temporary results
%                  folders.(see set_temp_result_folders.m)

% Author: Devin Sullivan 6/13/13
%
% Copyright (C) 2007-2013 Murphy Lab
% Carnegie Mellon University
%
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu/ or
% send email to murphy@cmu.edu

% Gregory Johnson 6/17/13
%   Modified for single image
%   Shortened some variable names 
%   Formatted for readability
%
% Xiongtao Ruan 01/06/2016
%   change seg_cell_and_nucleus_2D to seg_cell_and_nucleus_3D. 
%   add the diffeomorphic case of parameter setting.
%
% Ivan E. Cao-Berg 04/15/2016
%   Fixed bug where image being downsampled twice.

options = ml_initparam(options, struct('downsampling', [1,1,1], ...
                                        'display', false, ...
                                        'train', []));                                  

options.train = ml_initparam(options.train, struct('flag', 'all'));

options.model.resolution_orig = options.model.resolution;
options.model.resolution = options.model.resolution.*options.downsampling;
                                    
if options.verbose
    disp(['Loading images with downsampling set to: [' ...
        num2str(options.downsampling) ']']);
end
imdna = loadImage(imdna_path, options.downsampling);
imcell = loadImage(imcell_path, options.downsampling);
improt = loadImage(improt_path, options.downsampling);
immask = loadImage(immask_path, options.downsampling);

savefile = [savedir filesep 'seg.mat'];
if ~exist(savefile, 'file')
    % xruan 01/06/2016
    % change seg_cell_and_nucleus_2D to seg_cell_and_nucleus_3D
%    [seg_dna, seg_cell] = seg_cell_and_nucleus_2D( ...
    tic;
    [seg_dna, seg_cell] = seg_cell_and_nucleus_3D( ...
        imdna, ...
        imcell, ...
        improt, ...
        immask, ...
        [1 1 1], ...   %% xruan 01/06/2015 this is just set to be compatible with other cases. 
        options);
    options.running_time.segmentation = toc;
    
    seg.nuc = seg_dna;
    seg.cell = seg_cell;
    
    save(savefile, 'seg')
else
    load(savefile)
end
param.seg = seg;

%Compute nuclear image feats
savefile = [savedir filesep 'nuc.mat'];
if ~isempty(imdna) && ~exist(savefile, 'file')
    tic
    switch options.nucleus.type
        case 'cylindrical_surface'
            [spfeat,surfmap] = tp_nucimgfeat(param.seg.nuc, ...
                         'cylsurf', options );
                     
            nuc.spfeat = spfeat;
            nuc.surfmap = surfmap;
        % xruan 01/06/2016    
        case 'diffeomorphic'
            nuc.type = 'diffeomorphic';
            nuc.seg = param.seg.nuc;
        otherwise
            warning(['Unsupported nuclear model type ' options.nucleus.type '. Returning empty model.'])
            nuc = [];
    end
    
    save(savefile, 'nuc')
    options.running_time.nuclear_image_parameterization = toc;
else
    load(savefile)
end
param.nuc = nuc;

% xruan 03/08/2016 change cell to cellfit to unify name and resolve
% conflition with matlab built-in function
%Now get the Cell level parameters
savefile = [savedir filesep 'cell.mat'];
if ~isempty(imcell)
    if ~exist(savefile, 'file')
        tic
        switch options.cell.type
            case 'ratio'
        
                cellfit = cellfit_percell( ...
                        param.seg.nuc,  ...
                        param.seg.cell, ...
                        options);
            % xruan 01/06/2016 
            case 'diffeomorphic'
                cellfit.type = 'diffeomorphic';
                cellfit.seg = param.seg.cell;

            otherwise
                warning(['Unsupported cell model type ' options.nucleus.type '. Returning empty model.'])
                cellfit = [];
        end

        save(savefile, 'cellfit')
        options.running_time.cell_image_parameterization = toc;
    else
        load(savefile)
    end
    param.cell = cellfit;
end

%Now compute the protein models.
if ~isempty(improt) && strcmpi( options.train.flag, 'all' )
    savefile = [savedir filesep 'prot.mat'];
    if exist(savefile, 'file')
        load(savefile)
    else
        tic
        switch options.protein.type
            case 'gmm'
                prot = protfit_percell_3D(param.seg.nuc, ...
                        param.seg.cell, ...
                        improt, ...
                        param.seg.cell, ...
                        options);
            case 'microtubule_growth'
                savedir_mt = [savedir filesep 'mt_feats'];
                
                prot = img2microtubule_param_3D(improt, ...
                    param.seg.nuc, ...
                    param.seg.cell, ...
                    savedir_mt, ...
                    options);
            otherwise
                error(['Unrecognized protein model type ' options.protein.type])
        end
        save(savefile, 'prot')
        options.running_time.protein_image_parameterization = toc;
    end
    param.prot = prot;
end


savefile = [savedir filesep 'compartment.mat'];
%Lastly use any preprocessed results to create compartment
%models
if ~exist(savefile, 'file')
    compartment = compartment_percell(...
        param.seg.nuc, ...
        param.seg.cell,...
        improt, ...
        options);
    
    save(savefile, 'compartment')
else
    load(savefile)
end

param.compartment = compartment;
param.options = options;

end


    
