function param = img2param_2D( imdna_path,imcell_path,...
    improt_path,immask_path, savedir, options )

% Gregory Johnson
%
% Copyright (C) 2016 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

options = ml_initparam(options, struct('downsampling', [1,1], ...
    'display', false, ...
    'debug', false, ...
    'verbose', false, ...
    'train', []));

options.train = ml_initparam(options.train, struct('flag', 'all'));

options.model.resolution_orig = options.model.resolution;
options.model.resolution = options.model.resolution.*options.downsampling;

options = set_default_options(options);

imdna = loadImage(imdna_path, options.downsampling);
imcell = loadImage(imcell_path, options.downsampling);
improt = loadImage(improt_path, options.downsampling);
immask = loadImage(immask_path, options.downsampling);

param.imsize = size(imdna);

%Do the segmentation
savefile = [savedir filesep 'seg.mat'];
if ~exist(savefile, 'file')
    
    [seg_dna, seg_cell] = seg_cell_and_nucleus_2D( ...
        imdna, ...
        imcell, ...
        immask, ...
        options);
    
    seg.nuc = seg_dna;
    seg.cell = seg_cell;
    
    save(savefile, 'seg')
else
    load(savefile)
end
param.seg = seg;

%Compute Nuclear image feats
% xruan 01/05/2016 add the case for diffeomorphic
savefile = [savedir filesep 'nuc.mat'];
if ~isempty(imdna)
    
    options.nucleus = ml_initparam(options.nucleus, struct('type', 'medial axis'));
    
    if exist(savefile, 'file')
        load(savefile)
    else
        switch options.nucleus.type
            case 'medial axis'
                nuc = img2medial_axis(param.seg.nuc);
            case 'diffeomorphic'
                nuc.type = 'diffeomorphic';
                nuc.seg = param.seg.nuc;
            otherwise
                error(['Unrecognized nuclear shape model type ' options.protein.type])
        end
        
        save(savefile, 'nuc');
    end
else
    load(savefile)
end
% xruan 03/08/2016 change nucleus to nucleus to unify the names with 3D
% case
param.nuc = nuc;

%Now get the Cell level parameters
% xruan 01/05/2016 add the case for diffeomorphic
savefile = [savedir filesep 'cell.mat'];
if ~isempty(imcell)
    
    options.cell = ml_initparam(options.cell, struct('type', 'ratio'));
    
    if ~exist(savefile, 'file')
        switch options.cell.type
            case 'ratio'
                options.nuclear_image_filename = imdna_path;
                options.cell_image_filename = imcell_path;
                cellfit = ml_parsecell2D({},ml_findmainobj2d_bw(param.seg.cell),ml_findmainobj2d_bw(param.seg.nuc),1,param.imsize,...
                    {'da','nucarea','nuccenter','nucmangle','nuchitpts',...
                    'nuccontour','nucellhitpts','nucdist','nucelldist','nucecc',...
                    'cellarea','cellcenter','cellmangle','cellcontour',...
                    'cellhitpts','celldist','cellecc'}, options );
            case 'diffeomorphic'
                cellfit.type = 'diffeomorphic';
                cellfit.seg = param.seg.cell;
            otherwise
                error(['Unrecognized nuclear shape model type ' options.protein.type])
        end
        save(savefile, 'cellfit')
    else
        load(savefile)
    end
    param.cell = cellfit;
end


savefile = [savedir filesep 'preprocess.mat'];
tic
if ~isempty(improt) && ~exist(savefile, 'file')
    if options.debug && options.display
        imshow( improt );
        title( imdna )
        %interactive_display(options);
    end
    
    options = ml_initparam(options, struct(...
        'threshmeth_prot', 'nih'));
    procimage = ...
        ml_preprocess(double(improt),immask,'ml','yesbgsub',options.threshmeth_prot);
    
    save(savefile, 'procimage')
elseif exist(savefile, 'file')
    load(savefile)
else
    procimage = [];
end
param.prot.procimage = procimage;


%Now compute the protein models.
if ~isempty(improt)
    savefile = [savedir filesep 'prot.mat'];
    
    options.protein = ml_initparam(options.protein, struct('type', 'vesicle'));
    
    if exist(savefile, 'file')
        load(savefile)
    else
        switch options.protein.type
            case 'vesicle'
                if ~isfield(options, 'ml_objgaussmix')
                    options.ml_objgaussmix = [];
                end
                prot = img2protfit_2D(procimage, options.ml_objgaussmix);
                
            case 'network'
                %%% do this for the microtubule model %%%
            otherwise
                error(['Unrecognized protein model type ' options.protein.type])
        end
        save(savefile, 'prot')
    end
    param.prot = prot;
end


param.options = options;
end

function options = set_default_options(options)
options = ml_initparam(options, struct('cell', [], ...
    'nucleus', [], ...
    'protein', []));

if ~isfield(options.nucleus, 'type') || isempty(options.nucleus.type)
    options.nucleus.type = 'medial axis';
end

if ~isfield(options.cell, 'type') || isempty(options.cell.type)
    options.cell.type = 'ratio';
end

if ~isfield(options.protein, 'type') || isempty(options.protein.type)
    options.protein.type = 'vesicle';
end



end





%
%
% %given an image, returns a cellOrganizer parameterization
%
% options = ml_initparam(options, struct('downsampling', [1,1], ...
%                                         'display', false, ...
%                                         'train', []));
%
% options.train = ml_initparam(options.train, struct('flag', 'all'));
%
% options.model.resolution = options.model.resolution./options.downsampling;
%
%
% %Initialize options
% options = get_cellorganizer_default_parameters( options );
% %D. Sullivan 6/5/13 imageSize should not be hard coded, need to fix this.
% %also, why do we have both param.display and param.disp?
% options = ml_initparam(options, ...
%     struct('modelname','vesicle'...
%     ,'display',0 ...
%     ,'threshmeth_dna', 'nih' ...
%     ,'threshmeth_prot', 'nih' ...
%     ,'ml_objgaussmix', []));
%
% %icaoberg - changed isshow to 0
% options.ml_objgaussmix = ml_initparam(options.ml_objgaussmix, ...
%     struct('filter',fspecial('gaussian',10,10),'mindist',10, ...
%     'isshow',0, 'gmm',struct('covartype','spherical',...
%     'options',[0 1 0 0 0 0 0 0 0 0 0 0 0 500])));
%
% %if these three flags are set to true that means the user would like to
% %save the figure shown during debugging
% if options.debug && options.display && options.save_helper_figures
%     helper_figures_folder = [savedir filesep 'figures'];
%     subfolders = { 'nucleus', 'cell', 'protein' };
%     for s=1:1:length(subfolders)
%         subfolder = [helper_figures_folder filesep subfolders{s} ];
%         if ~exist( subfolder, 'dir' )
%             mkdir( subfolder )
%         end
%     end
% end
% clear subfolders
% clear s
%
% %Preprocessing
% procfile = [savedir filesep 'preprocessed.mat'];
% if ~exist(procfile, 'file')
%     disp('Processing image')
%     if ~isempty(immask_path)
%         maskimg = ml_readimage(immask_path);
%     else
%         maskimg = [];
%     end
%
%     %Preprocessing nuclear image
%     disp('Preprocessing nuclear image');
%     %read image
%     tic
%     img = ml_readimage(imdna_path);
%     if isempty(img)
%         disp('Unable to load image. Skipping.')
%     end
%     %preprocessing
%     procimg = ml_preprocess(double(img),maskimg,'ml','yesbgsub', options.threshmeth_dna);
%     nucbodyimg = imfill(procimg>0,'hole');
%     %edge detection (use tz_imedge)
%     %   nucedge = bwperim(nucbodyimg);
%     nucbody = ml_findmainobj_bw(nucbodyimg);
%     toc
%
%     %Preprocessing cell image
%     disp('Preprocessing cell image');
%     if ~isempty( imcell_path )
%         tic
%         %read cell image
%         img = ml_readimage(imcell_path);
%         if isempty( img )
%             disp('Unable to load image. Skipping.');
%         end
%
%         if ~isempty(maskimg)
%             img(maskimg==0) = 0;
%         end
%
%         if length(unique(img(:))) > 2
%             celledge = ml_imedge(img,[],'ce');
%             cellbodyimg = imfill(celledge,'hole');
%         else
%             cellbodyimg = img;
%         end
%
%         cellbody = ml_findmainobj_bw(cellbodyimg);
%         toc
%     else
%         cellbodyimg = [];
%         cellbody = [];
%     end
%     cellbody = ml_findmainobj_bw(cellbodyimg);
%     toc
% %{
%     display_choices.proc_img_choice = cellbodyimg;
%     display_choices.raw_img_choice = imcell;
%     display_choices.name_choice = 'cell';
%     display_choices.title_choice = 'Found cell body';
%
%     display_img(options, img, imdna, display_choices);
% %}
%
%     %Preprocessing protein image
%     disp('Preprocessing protein image');
%     tic
%     if ~isempty(improt_path)
%         img = ml_readimage(improt_path);
%         if isempty( img )
%             disp('Unable to load image. Skipping.');
%         end
%         %icaoberg 5/20/2013
%         if options.debug && options.display
%             imshow( img );
%             title( imdna )
%             interactive_display(options);
%         end
%         procimage = ...
%             ml_preprocess(double(img),maskimg,'ml','yesbgsub',options.threshmeth_prot);
%     else
%         procimage = [];
%     end
%     toc
%
% %{
%     display_choices.proc_img_choice = procimage;
%     display_choices.raw_img_choice = improt;
%     display_choices.name_choice = 'protein';
%     display_choices.title_choice = 'Found protein pattern';
%
%     display_img(options, img, imdna, display_choices);
% %}
%
%     %save results to disc
%     save( procfile, 'nucbody', 'cellbody', ...
%         'nucbodyimg', 'cellbodyimg', 'procimage', 'maskimg', 'img' );
%     close all
% else
%     disp('Preprocessed results found. Skipping recalculation.');
%     load( procfile );
% end
% param.procfile = procfile;
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% savefile = [savedir filesep 'seg.mat'];
% if ~exist(savefile, 'file')
%     seg.dna = nucbody;
%     seg.cell = cellbody;
%     save(savefile, 'seg')
% else
%     load(savefile)
% end
% param.seg = seg;
%
% %Compute Nuclear image feats
% disp('Extracting medial axis')
% savefile = [savedir filesep 'nuc.mat'];
% if ~exist(savefile, 'file')
%     tic
%     %rotate nuclear image
%     theta = ml_majorangle(nucbodyimg)*180/pi;
%     nucbodyimg = ml_rotate(nucbodyimg,-theta);
%
%     %extract medial axis
%     [imgaxis, medaxis, width] = ...
%         ml_imaxis(uint8(nucbodyimg));
%
%
%     shape = ml_mxs2mxp(medaxis,width);
%     nuc.shape = shape;
%     save( savefile, 'nuc' )
%     toc
% else
%     disp('Medial axis found for image. Skipping recalculation.');
%     tic
%     shape = load( savefile );
%     nuc.shape = shape;
%     toc
% end
% param.nuc = nuc;
%
% %Now get the Cell level parameters
% savefile = [savedir filesep 'cell.mat'];
% if ~strcmpi(options.train.flag,'nuc')
%     if ~exist(savefile, 'file')
%         disp('Calculation of cell object statistics on image ');
%         tic
%         load(procfile)
%         %icaoberg 5/13/2013
%         try
%             anglestep = options.model.anglestep;
%         catch
%             anglestep = 1;
%         end
%
%         options.nuclear_image_filename = imdna_path;
%         options.cell_image_filename = imcell_path;
%         cellcode = ml_parsecell2D({},cellbody,nucbody,anglestep,size(img),...
%             {'da','nucarea','nuccenter','nucmangle','nuchitpts',...
%             'nuccontour','nucellhitpts','nucdist','nucelldist','nucecc',...
%             'cellarea','cellcenter','cellmangle','cellcontour',...
%             'cellhitpts','celldist','cellecc'}, options );
%         save(savefile, 'cellcode');
% %         options = rmfield( options, 'image_index' );
%         options = rmfield( options, 'cell_image_filename' );
%         options = rmfield( options, 'nuclear_image_filename' );
%         toc
%     else
%         disp('Cell codes found for image. Skipping recalculation.');
%         cellcode = load( savefile );
%     end
%     cell.cellcode = cellcode;
%     param.cell = cell;
% end
%
% if strcmpi( options.train.flag, 'framework' )
%     return
% end
%
% %Now compute the protein models.
% if strcmpi(options.train.flag,'all')
%     savefile = [savedir filesep 'prot.mat'];
%     latents = [];
%     intensities = [];
%     if ~exist(savefile, 'file')
%         disp('Learning Gaussian mixtures');
%         tic
%         load(procfile)
%         objs = ml_findobjs2D(procimage);
%         toc
%
%         for j=1:length(objs)
%             disp(['Object:' num2str(j)]);
%             obj = objs{j};
%             objintens(j) = sum(obj(:,3));
%             if size(obj,1)==1
%                 mixes{j} = [];
%             else
%                 tic
%                 mixture = ml_objgaussmix2D( obj,options.ml_objgaussmix );
%                 mixes{j} = mixture;
%
%                 if options.ml_objgaussmix.isshow == 1
%                     drawnow
%                 end
%
%                 toc
%                 if ~isempty(improt_path)
%                     %Get positions
%                     objcof = [];
%                     objintensity = objintens(j);
%                     if ~isempty(mixes{j})
%                         for k=1:mixes{j}.ncentres
%                             latents = [latents; mixes{j}.covars(1,1,k)];
%                             intensities = [intensities; ...
%                                            objintensity*mixes{j}.priors(k)];
%                             objcof(end+1,:) = mixes{j}.centres(k,:);
%                         end
%                     end
%                 end
%             end
%         end
%         %object number statistics
%         objnums = [];
%         for j=1:length(objcof)
%             objnums(:) = size(objcof(j),1);
%         end
%
%         %remove illegal entries
%         rminds = isnan(latents) | isnan(intensities);
%         latents(rminds) = [];
%         intensities(rminds) = [];
%
%         prot.objs = objs;
%         prot.mixes = mixes;
%         prot.latents = latents;
%         prot.intensities = intensities;
%         prot.objcof = objcof;
%         prot.objnums = objnums;
%         save(savefile, 'prot');
%     else
%         disp('Preprocessed mixture found for image  object .');
%         load(savefile);
%     end
%     param.prot = prot;
% end
%
% end
%
% function interactive_display(options)
%     if isfield( options, 'interactive' ) && ...
%             options.interactive
%         waitforbuttonpress;
%     else
%         if ~isfield( options, 'time_to_pause' )
%             options.time_to_pause = 2;
%         end
%         pause( options.time_to_pause );
%     end
% end
%
% %this display the raw image and the preprocessed image of choice
% function display_img(options, img, imdna, display_choices)
%     if options.debug && options.display
%         %this is done to open a maximized figure
%         screen_size = get(0, 'ScreenSize');
%         f1 = figure;
%         set(f1, 'Position', [0 0 screen_size(3) screen_size(4)]);
%
%         %make and label plots
%         subplot(1,2,1);
%         imshow( img );
%         title('Raw image');
%         subplot(1,2,2);
%         imshow( display_choices.proc_img_choice );
%         title(display_choices.title_choice);
%         mtit( imdna )
%
%         %save helper figures
%         if isfield( options, 'save_helper_figures' ) && ...
%                 options.save_helper_figures
%             subfolder = [ helper_figures_folder filesep display_choices.name_choice ];
%             [path, filename, extension] = fileparts( display_choices.raw_img_choice );
%             print( gcf, '-dtiff', [ subfolder filesep filename extension ]);
%         end
%         clear subfolder
%
%         interactive_display(options);
%     end
%     close
% end
%
%

